class Hermes::EntityReservation
  include Mongoid::Document

  include Hermes::Concerns::CommonFieldsConcern
  include Hermes::Concerns::ActivableConcern
  field :entity_id, type: BSON::ObjectId
  field :housing_offer_id, type: BSON::ObjectId
  field :name, type: String
  field :from, type: Date
  field :to, type: Date

  common_indexes *%i[
    entity_id
    housing_offer_id
    name
    from
    to
  ]

  belongs_to(:housing_offer, class_name: 'Hermes::HousingOffer')
  belongs_to(:entity, class_name: 'Hermes::Entity')
  
  validates_presence_of :entity_id, :housing_offer_id, :from, :to
  
  owner_field :user_id

  before_save :before_save_actions
  after_save :after_save_actions
  after_destroy :after_destroy_actions

  @force_is_active = false

  def self.update_is_active
    today = Date.today
    not_active = Hermes::EntityReservation.unscoped.where(is_active: true).or(from: { '$gt': today }).or(to: { '$lt': today })
    new_active = Hermes::EntityReservation.unscoped.where(is_active: false, from: { '$lte': today }, to: { '$gte': today } )

    not_active.each do |er|
      er.update(is_active: false)
    end
    new_active.each do |er|
      er.update(is_active: true)
    end 
  end 

  def check_if_active(date = DateTime.now())
    (self.from.beginning_of_day .. self.to.end_of_day).cover?(date)
  end
  
  def update_activity(date = DateTime.now())
    self.is_active = check_if_active(date)
    if self.is_active_changed?
      @force_is_active = true
      self.save 
      @force_is_active = false
    end
  end

  def before_save_actions
    self.is_active = check_if_active if !@force_is_active
    update_rights
    if entity_id_changed? && entity_id_was && housing_offer
      remove_reservation_from_offer(housing_offer, entity_id_was)
      if entity_id && is_active
        add_reservation_from_offer(housing_offer, entity_id)
      end 
    end  
    true
  end

  def after_save_actions
    add_reservation_from_offer(housing_offer, entity_id) if is_active_changed? && is_active
    remove_reservation_from_offer(housing_offer, entity_id) if is_active_changed? && is_active_was == true
  end

  def remove_reservation_from_offer(offer, e_id)
    offer.active_reservations.delete(e_id.to_s)
    offer.save
    create_status(false, e_id) if Hermes::Status.where(housing_offer: offer, entity_id: e_id).count
  end 

  def add_reservation_from_offer(offer, e_id)
    offer.active_reservations.push(e_id.to_s)
    offer.save
    create_status(true, e_id)
  end 

  def create_status(is_a, e_id)
    if entity = Hermes::Entity.find(e_id)
      Hermes::Status.create(
        reference_model_name: "HousingOffer",
        status_label: "entity_status",
        motive: is_a ? "Est entré en période de réservation" : "La période de réservation a prit fin",
        title: is_a ? "Réservé par #{entity.title}" : "Fin de réservation pour #{entity.title}",
        object_id: housing_offer_id,
        performer_id: e_id,
      )
    end 
  end 

  def after_destroy_actions
    remove_reservation_from_offer(housing_offer, entity.id)
  end 

  def update_rights
    if self.user_id.nil?
      self.user_id = entity.user_id
    end
  end

  
end
