# frozen_string_literal: trues

module Hermes
  # HousingIntention model
  class HousingApplication
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::Status
    include Mongoid::Attributes::Dynamic
    
    include Hermes::Concerns::SopraXmlSender
    field :applicated_at, type: DateTime
    field :effort_rate, type: Float
    field :epci, type: Integer
    field :is_active, type: Boolean, default: true
    field :max_date, type: DateTime
    field :visit_conducted_date, type: DateTime
    field :event_date, type: Date
    field :last_status_update_date, type: Date
    field :lessor_event_informations, type: String
    field :sopra_code, type: String
    field :als_id, type: String
    field :rank, type: Integer
    
    field :profile_epci_score, type: Integer
    field :eligible_position, type: Integer
    field :applicated_position, type: Integer
    field :effort_rate, type: Float
    field :score_id, type: String
    
    common_indexes *%i[
      applicated_at
      effort_rate
      epci
      is_active
      max_date
      visit_conducted_date
      last_status_update_date
      lessor_event_informations
      sopra_code
      rank
      profile_epci_score
      eligible_position
      applicated_position
      effort_rate
      score_id
      als_id
    ]
    
    belongs_to :housing_offer, index: true, class_name: 'Hermes::HousingOffer'
    belongs_to :housing_request, index: true, class_name: 'Hermes::HousingRequest'
    
    validates_presence_of :housing_request_id, :housing_offer, unless: -> {disabled_or_deleted_status?}
    validates :status, inclusion: {in: Hermes::Statuses::HousingApplication.list.values.flatten, allow_blank: true}
    validates :visit_status, inclusion: {in: Hermes::Statuses::HousingApplication.list[:visit], allow_blank: true}
    
    writer_auth_roles "collaborateur offre logement", 'superadministrateur'
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id
    
    before_validation :before_validation_actions
    before_save :before_save_actions
    after_save :after_save_actions
    after_create :after_create_actions
    after_update :after_actions
    after_destroy :after_destroy_actions
    
    scope :selected_by_offer, ->(housing_offer_id) {
      where(housing_offer_id: housing_offer_id, status: "TRANSMIS BAILLEUR")
    }
    
    scope :actives, ->() {where(is_active: true)}
    
    HousingApplication.add_status(:single)
    HousingApplication.add_status(:single, "visit_status")
    
    INACTIV_STATUS = [
      'DESISTEMENT APRES CAL',
      'REFUSEE EN CAL',
      'NON RETENUE',
      'NON RETENUE EN CAL',
      'LOGE',
      'DESISTEMENT SALARIE AVANT CAL',
      'ANNULEE BAILLEUR',
      'DESISTEMENT',
      'ANNULEE',
    ]
    
    INACTIV_VISIT_STATUS = [
      'DESISTEMENT APRES VISITE',
    ]

    SEND_CANCEL_STATUS = [
      'DESISTEMENT APRES VISITE',
      'DESISTEMENT SALARIE AVANT CAL',
      'DESISTEMENT',
      'DESISTEMENT APRES CAL',
      'ANNULEE',
    ]
    
    ###############################
    #### Notification #############
    include ::Concerns::NotificationCenter::Notifiable
    
    def self.removeScoreData(housing_application_ids)
      return unless housing_application_ids.length
      Hermes::HousingApplication.where(:id.in => housing_application_ids).set(
        profile_epci_score: nil,
        eligible_position: nil,
        applicated_position: nil,
        effort_rate: nil,
        score_id: nil,
        updated_at: DateTime.now
      )
    end

    def self.targets_selector(housing_app, opt = {})
      [housing_app.applicant].compact
    end
    
    def notification_mailer
      ::Hermes::NotificationCenter::HermesMailer
    end
    
    def notifiable_title(key, opt = {})
      I18n.t("#{key}.title")
    end
    
    def notifiable_body(key, opt = {})
      I18n.t("#{key}.body", number: housing_offer.reference, fullname: applicant.full_name)
    end
    
    def notifiable_path
      "https://als-hermes-int.be-idys.com/housing_applications/#{id}"
    end
    
    ###############################
    ###############################
    
    def to_json(options = {})
      options[:except] ||= :sopra_code
      super
    end
    
    def as_json(options = {})
      options[:except] ||= :sopra_code
      super
    end
    
    def applicant
      housing_request&.main_applicant
    end
    
    def applicant_id
      applicant&.id
    end
    
    def update_offer_applicated_nb
      return unless housing_offer
      housing_offer.reload.update(applicated_nb: Hermes::HousingApplication.where(
        is_active: true, 
        housing_offer_id: housing_offer_id,
      ).count)
    end

    def set_als_id
      self.als_id = housing_request&.als_id
    end

    def als_id
      value = super
      return value if value.present?

      if housing_request
        value = housing_request.als_id
        set(als_id: value, updated_at: DateTime.current)
        self.als_id = value
        value
      end
    end

    def handle_status_changed
      applicant_cancel_application if disabled_or_deleted_status?
     
      if status&.upcase == 'ACCEPTEE EN CAL'
        applicant.completed_requests_nb ? (applicant.completed_requests_nb += 1) : (applicant.completed_requests_nb = 1)
        applicant.save
        housing_request.update_applicant_eligbility
      end
      
      code_to_send = nil
      if status_changed?
        code_to_send = Hermes::Statuses::HousingApplication.sopra_code(status)
        create_public_share if status == 'TRANSMIS BAILLEUR'
      end
      if visit_status_changed?
        case visit_status
        when 'VISITE PROPOSEE'
          code_to_send = "VISPRO"
        when 'VISITE EFFECTUEE'
          code_to_send = "VISEFF"
        end
      end
      
      return unless code_to_send && !code_to_send.blank?
      so = Hermes::BoClient::UpdateStatusTask.new({
                                                    housing_application_id: id,
                                                    housing_request_id: housing_request_id,
                                                    housing_offer_id: housing_offer_id,
                                                    code_evenement: code_to_send,
                                                    date_evenement: last_status_update_date || Date.current,
                                                    date_survenance_evenement: event_date,
                                                    rang: rank 
                                                  })
      if Rails.env.test?
        #{exectuted: true, xml: so.to_xml}
        true
      else
        so.execute
      end
    end
    
    def handle_visit_conducted_date
      if visit_conducted_date
        if visit_conducted_date.past?
          self.set(visit_status: 'Visite effectuée', updated_at: DateTime.current) if visit_status.blank?
        else
          self.set(visit_status: 'Visite proposée', updated_at: DateTime.current) if visit_status.blank?
        end
      end
    end

    def update_user_id_from_request
      self.user_id = housing_request.user_id
    end 
    
    def before_save_actions
      update_application_score
      update_user_id_from_request if housing_request_id_changed? && housing_request_id
      change_is_active if status_changed? || visit_status_changed?
      update_reader_writers_from_offer if housing_offer_id_changed?
      set_als_id if housing_request_id_changed?
      true
    end

    def update_application_score
      score = Hermes::Score.find_by(is_active: true, applicant: housing_request.main_applicant, housing_offer: housing_offer)

      self.profile_epci_score = score ? score.profile_epci_score : nil
      self.eligible_position = score ? score.eligible_position : nil
      self.applicated_position = score ? score.applicated_position : nil
      self.effort_rate = score ? score.effort_rate : nil
      self.score_id = score ? score.id.to_s : nil
    end 
    
    def update_reader_writers_from_offer
      old_offer = Hermes::HousingOffer.find(housing_offer_id_was) if housing_offer_id_was
      
      if housing_offer_id_was && old_offer
        self.readers = (self.readers || []) - (old_offer.readers || [])
        self.writers = (self.writers || []) - (old_offer.writers || [])
      end
      
      if housing_offer
        self.readers = (self.readers || []) + (housing_offer.readers || [])
        self.writers = (self.writers || []) + (housing_offer.writers || [])
      end
    end
    
    def change_is_active
      self.is_active = !(INACTIV_STATUS.include?(self.status) || INACTIV_VISIT_STATUS.include?(self.visit_status))
    end
    
    def before_validation_actions
      if status_changed?
        self.status_updated_at = DateTime.current
        self.sopra_code = Hermes::Statuses::HousingApplication.sopra_code(status)
        #  self.status = Hermes::Statuses::HousingApplication.correspondence(status || '')
      end
      
      if visit_status_changed?
        self.visit_status_updated_at = DateTime.current
        self.visit_status = I18n.transliterate(visit_status || '').upcase
      end
      update_right
    end
    
    def reprise_update_right
      apps = Hermes::HousingApplication.where(is_active: true)
      apps.length
      apps.batch_size(5).each_with_index do |app, i|
        if app.housing_request
          app.update_right
          app.save
        end
      
      end
    end
    
    def update_right
      lessor_user_id = housing_offer&.lessor&.user_id&.to_s
      main_applicant = housing_request&.main_applicant
      reader_inst = [self, housing_request, main_applicant]
      if lessor_user_id && main_applicant
        if disabled_or_deleted_status?
          reader_inst.each do |inst|
            inst.pull(readers: lessor_user_id)
          end
          Hermes::Applicant.where(:id.in => housing_request.applicant_ids).pull(readers: lessor_user_id)
        else
          reader_inst.each do |inst|
            inst.add_to_set(readers: lessor_user_id)
          end
          Hermes::Applicant.where(:id.in => housing_request.applicant_ids).add_to_set(readers: lessor_user_id)
        end
        reader_inst.each do |inst|
          inst.add_to_set(readers: main_applicant.readers)
        end
      end
    end
    
    def only_one_active_application
      return unless housing_offer_id || housing_request_id
      Hermes::HousingApplication.where({
        :is_active => true,
        :housing_offer_id => housing_offer_id,
        :housing_request_id => housing_request_id,
        :id.ne => id
      }).each do |a|
        a.update(is_active: false, status: 'ANNULEE')
      end
    end
    
    def update_request_active_application_id
      if is_active
        housing_request.set(
          updated_at: DateTime.now,
          active_application_id: id.to_s,
          active_application_housing_offer_id: housing_offer_id.to_s
        )
      elsif housing_request.active_application_id.to_s == id.to_s
        housing_request.set(
          updated_at: DateTime.now,
          active_application_id: nil,
          active_application_housing_offer_id: nil
        )
      end 
    end
    
    def impact_offer_and_request_status
   
      case status
      when 'TRANSMIS BAILLEUR'
        
        housing_offer.lessor_status = "EN ATTRIBUTION"
        housing_offer.offer_status = "EN COURS D'ATTRIBUTION"
        housing_offer.reject_others_application
        housing_offer.save 
      when 'LOGE'
        housing_request.request_status = "SATISFAITE"
        housing_request.entity_status = "SALARIE LOGE"
        housing_request.save
       
        housing_offer.offer_status = "LOGEMENT PLACE"
        housing_offer.save
      when 'ACCEPTEE EN CAL'
        housing_request.request_status = "SATISFAITE"
        housing_request.entity_status = "SALARIE LOGE"
        housing_request.save
       
        #housing_offer.offer_status = "LOGEMENT PLACE"
        #housing_offer.save
      else
        true
      end
    
    end
    
    def after_save_actions
      # Broken tests
      check_status_changed if visit_status_changed? || status_changed?
      
      # housing_request&.with_lock(reload: false, wait: true) do
        housing_request.save
      # end
      # notify_applicant # Temporary_disabled_devops_7390
      handle_status_changed if visit_status_changed? || status_changed?
      send_flux_50 if status_changed? && (SEND_CANCEL_STATUS.include?(status) || SEND_CANCEL_STATUS.include?(visit_status))
      impact_offer_and_request_status if status_changed?
      only_one_active_application if is_active
      update_request_active_application_id if is_active_changed?
      housing_offer.update_eligibility_score if is_active_changed?
      update_offer_applicated_nb
    end
    
    def after_actions
      handle_visit_conducted_date
    end
    
    def after_create_actions
      update_offer_applicated_nb
      housing_request.set(last_application_date: created_at, updated_at: DateTime.current)
      HandleRequestStateJob.set(wait: 1.days).perform_later(housing_request_id.to_s, 'request_application')
      
      if Rails.env == 'test'
        {exectuted: true, xml: to_xml}
      else
        Hermes::BoClient::ApplicationTask.new(
          {
            housing_application_id: id,
            housing_request_id: housing_request_id,
            housing_offer_id: housing_offer_id,
          }
        ).execute
      end
    end
    
    def after_destroy_actions
      # housing_request.with_lock(reload: false) do
        housing_request.save
      # end
      update_offer_applicated_nb
    end
    
    def disabled_or_deleted_status?
      INACTIV_STATUS.include?(self.status) || INACTIV_VISIT_STATUS.include?(self.visit_status)
    end

    def send_flux_50
      so = Hermes::BoClient::CancelStatusTask.new({
                                                    housing_application_id: id,
                                                    housing_offer_id: housing_offer_id,
                                                    housing_request_id: housing_request_id,
                                                    status_updated_at: status_updated_at || DateTime.now
                                                  })
      if Rails.env.test?
        #{exectuted: true, xml: so.to_xml}
        so.execute
        true
      else
        so.execute
      end
    end 
    
    def applicant_cancel_application
      housing_request.update_applicant_eligbility
      housing_request.save
    end
    
    def system_cancel_application!
      set(status: 'ANNULEE', is_active: false, updated_at: DateTime.current)
      housing_request.update_applicant_eligbility
      housing_request.cancel_request
    end

    def create_public_share
      return nil unless housing_offer.lessor
      Hermes::HousingRequestShare.find_or_create_by(lessor_id: housing_offer.lessor.id, housing_request_id: housing_request.id)
    end
    
    def notify_applicant
      if status_changed?
        case status.upcase
        when 'ACCEPTEE EN CAL'
          # notify lessor
          # notify(key: NOTIFICATION_KEYS[:housing_application][:lessor_accepted])
        when 'LOGE'
          notify(key: NOTIFICATION_KEYS[:housing_application][:lessor_accepted], channels: ['email'])
        when 'TRANSMIS BAILLEUR'
          notify(key: NOTIFICATION_KEYS[:housing_application][:lessor_transferred], channels: ['email'])
        when 'REFUSEE EN CAL', 'NON RETENUE', 'NON RETENUE EN CAL'
          # notify(key: NOTIFICATION_KEYS[:housing_application][:lessor_denied], channels: ['email'])
          notify(key: NOTIFICATION_KEYS[:housing_application][:cal_denied], channels: ['email'])
        when 'EN ATTENTE'
          notify(key: NOTIFICATION_KEYS[:housing_application][:pending], channels: [])
        when 'INSCRIPTION CAL'
          notify(key: NOTIFICATION_KEYS[:housing_application][:subscribed], channels: [])
        when 'DESISTEMENT SALARIE AVANT CAL', 'DESISTEMENT APRES CAL', 'DESISTEMENT'
          notify(key: NOTIFICATION_KEYS[:housing_application][:applicant_cancelled], channels: [])
        else
          nil
        end
      end
    end
  end
end
