require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  ACCEPTABLE_PARAMS = %w[
    type
    thresholds_map
    housing_offer_ids
  ].freeze

  THRESHOLD_MAP = {
    'Paris et alentours' => {
      'thresholds_list' => [
        '12673',
        '14679'
      ],
      'another_person' => '6723'
    }
  }.freeze

  describe Api::Dmo::ThresholdsController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper
    let!(:responsible) { create(:user, roles: ['Administrator']) }
    let!(:user) { create(:user) }

    #TESTS BASIQUES D'API

    describe 'POST /thresholds' do
      describe 'Create threshold with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                      user_id: responsible.id.to_s,
                      thresholds_map: THRESHOLD_MAP,
                      type: 'ceils'
                  }
              }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
          expect(json['data']['attributes']['thresholds_map']).to eq(THRESHOLD_MAP.all_to(Integer))
          expect(json['data']['attributes']['type']).to eq('ceils')
        end
      end
    end

    describe 'GET /thresholds' do
      describe 'GET /thresholds' do
        let!(:threshold1) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'ceils')}
        let!(:threshold3) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'ceils')}
        let!(:threshold2) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'ceils')}
        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end

      describe 'GET /thresholds with batch of ids' do
        let!(:threshold1) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'ceils')}
        let!(:threshold3) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'ceils')}
        let!(:threshold2) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'ceils')}
        before :each do
          authenticated_header(request, responsible)
          get 'index', {ids: [threshold1.id.to_s, threshold2.id.to_s]}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(2)
        end
      end
      describe 'GET /thresholds with operators' do
        let!(:threshold1) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'acceuillant')}
        let!(:threshold3) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'ceils')}
        let!(:threshold2) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'Seils')}

        describe 'with $in' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {type: {'$in': ['acceuillant', 'flamand']}}
          end
          it 'returns results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
        describe 'with $ne' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {type: {'$ne': 'acceuillant'}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(2)
          end
        end
        describe 'with $regex' do
          before :each do
            authenticated_header(request, responsible)
            get 'index', {type: {'$regex': /^S/}}
          end
          it 'returns sorted results' do
            expect(response.status).to eq(200)
            expect(json['meta']['pagination']['total_objects']).to eq(1)
          end
        end
      end
    end

    describe 'GET /thresholds/:id' do
      describe 'verif it work' do
        let!(:threshold1) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'acceuillant')}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: threshold1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:threshold1) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'acceuillant')}

        before :each do
          authenticated_header(request, user)
          get 'show', {id: threshold1.id}
        end
        it 'returns status code 404' do
          expect(response).to have_http_status(404)
        end
      end
    end

    describe 'PATCH /thresholds/:id' do

    end

    describe 'DELETE /thresholds/:id' do
      let!(:threshold) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'acceuillant')}
      before :each do
        authenticated_header(request, responsible)

        delete 'destroy', {
            id: threshold.id.to_s
        }
      end
      it 'returns status code 204' do
        expect(response).to have_http_status(204)
      end
    end

  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::ThresholdsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_THRESHOLDS = 'API_DMO_THRESHOLDS'.freeze
    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

    context 'as admin' do
      let!(:admin) {create(:administrator)}
      let!(:responsible) {create(:user, roles: ['Administrator'])}
      let!(:user) {create(:user)}

      path '/api/dmo/thresholds' do
        post 'Create threshold' do
          tags TAGS_API_DMO_THRESHOLDS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          json_params[:user_id] = {type: :string, required:true}
          json_params[:thresholds_map] = {type: :object, required:true}
          json_params[:type] = {type: :string, required:true}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )

          response '201', 'threshold created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes(
                  user_id: responsible.id.to_s,
                  thresholds_map: THRESHOLD_MAP,
                  type: 'acceuillant'
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'thresholds'
            end
          end
        end
      end

      path '/api/dmo/thresholds' do
        get 'Retrieves all thresholds' do
          tags TAGS_API_DMO_THRESHOLDS
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          #VALID_OPERATORS.each do |valid_operator|
          # parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          # end

          security [Bearer: {}]
          paginator

          response '200', 'thresholds found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:thresholds) {create_list(:threshold, 5, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'acceuillant')}

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {thresholds}
            end
          end
        end
      end

      path '/api/dmo/thresholds/{id}' do
        get 'Retrieves a single thresholds by id' do
          tags TAGS_API_DMO_THRESHOLDS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'thresholds found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:threshold) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'acceuillant')}
            let(:id) {threshold.id}

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {threshold}
            end
          end
        end

        put 'edit thresholds/{id}' do
          tags TAGS_API_DMO_THRESHOLDS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          json_params[:user_id] = {type: :string}
          json_params[:thresholds_map] = {type: :object}
          json_params[:type] = {type: :string}
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )

          response '200', 'threshold updated' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:threshold) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'acceuillant')}
            let(:id) {threshold.id}
            let(:params) do
              jsonapi_attributes(
                  id: id,
                  type: 'bibi'
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'thresholds'
            end
          end
        end

      end

      path '/api/dmo/thresholds/{id}' do
        delete 'Delete a single thresholds by id' do
          tags TAGS_API_DMO_THRESHOLDS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'thresholds deleted' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:id) {create(:threshold, user_id: responsible.id.to_s, thresholds_map: THRESHOLD_MAP, type: 'acceuillant').id}
            run_test!
            #include_context 'not containings errors'
          end
        end
      end
    end
  end
end
