require 'rails_helper'

module Hermes
  require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

  MUTATE_PARAMS = {
      insee_code: 'String',
      department: 'String',
  }

  ACCEPTABLE_PARAMS = %w[
  ids
          insee_code
          department
        ]

  VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $eq
          $regex
          $all
        ]

  describe Api::Dmo::UntensedAreasController, :type => :api do
    routes {Hermes::Engine.routes}
    include Devise::Test::ControllerHelpers
    include JwtAuthHelper
    let!(:responsible) {create(:user, roles: ['Administrator'])}
    let!(:user) {create(:user)}

    #TESTS BASIQUES D'API

    describe 'POST /untensed_areas' do
      describe 'Create untensed_area with minimal valid datas' do
        before :each do
          authenticated_header(request, responsible)
          post 'create', {
              data: {
                  attributes: {
                  }
              }
          }
        end
        it 'returns status code 201' do
          expect(response).to have_http_status(201)
        end
      end
      
      
    end

    describe 'GET /untensed_areas' do
      describe 'GET /untensed_areas' do
        let!(:untensed_area1) {create(:untensed_area)}
        let!(:untensed_area3) {create(:untensed_area)}
        let!(:untensed_area2) {create(:untensed_area)}

        before :each do
          authenticated_header(request, responsible)
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(3)
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(3)
        end
      end

      describe 'GET /untensed_areas with batch of ids' do
        let!(:untensed_area1) {create(:untensed_area)}
        let!(:untensed_area3) {create(:untensed_area)}
        let!(:untensed_area2) {create(:untensed_area)}

        before :each do
          authenticated_header(request, responsible)
          get 'index', {ids: [untensed_area2.id.to_s, untensed_area3.id.to_s]}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(2)
        end
      end
    end

    describe 'GET /untensed_areas/:id' do
      describe 'verif it work' do
        let!(:untensed_area1) {create(:untensed_area)}
        before :each do
          authenticated_header(request, responsible)
          get 'show', {id: untensed_area1.id}
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
        end
      end
      describe 'verif it work not work with a non authorized user' do
        let!(:untensed_area1) {create(:untensed_area)}

        before :each do
          authenticated_header(request, user)
          get 'show', {id: untensed_area1.id}
        end
        it 'returns status code 401' do
          expect(response).to have_http_status(401)
        end
      end
    end

    describe 'PATCH /untensed_areas/:id' do

    end

    describe 'DELETE /untensed_areas/:id' do
      let!(:untensed_area) {create(:untensed_area, user_id: responsible.id.to_s)}
      before :each do
        authenticated_header(request, responsible)

        delete 'destroy', {
            id: untensed_area.id.to_s
        }
      end
      it 'returns status code 204' do
        expect(response).to have_http_status(204)
      end
    end

  end

  require_relative '../../swagger_helper'

  describe Hermes::Api::Dmo::UntensedAreasController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
    TAGS_API_DMO_UNTENSED_AREAS = 'API_DMO_UNTENSED_AREAS'

    include JwtAuthHelper
    extend HermesSwaggerJsonParams
    include SwaggerJsonAttributes
    let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
    let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

    context 'as admin' do
      let!(:admin) {create(:administrator)}

      path '/api/dmo/untensed_areas' do
        post 'Create untensed_area' do
          tags TAGS_API_DMO_UNTENSED_AREAS
          produces 'application/json'
          consumes 'application/json'
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )

          response '201', 'untensed_area created' do
            let(:Authorization) {"Bearer #{token_for(responsible)}"}
            let(:params) do
              jsonapi_attributes(
                 {})
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'untensed_areas'
            end
          end
        end
      end

      path '/api/dmo/untensed_areas' do
        get 'Retrieves all untensed_areas' do
          tags TAGS_API_DMO_UNTENSED_AREAS
          produces 'application/json'

          parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
          ACCEPTABLE_PARAMS.each do |acceptable_param|
            type = ''
            if MUTATE_PARAMS.include?(acceptable_param)
              type = nice_type(MUTATE_PARAMS[acceptable_param])
            end
            parameter name: acceptable_param.to_s, in: :query, type: type, required: false
          end
          # VALID_OPERATORS.each do |valid_operator|
          #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
          # end

          security [Bearer: {}]
          paginator

          response '200', 'untensed_areas found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let!(:untensed_areas) {create_list(:untensed_area, 5)}

            include_context 'with integration test'
            include_context 'valid jsonapi collection' do
              let(:expected_collection) {untensed_areas}
            end
          end
        end
      end

      path '/api/dmo/untensed_areas/{id}' do
        get 'Retrieves a single untensed_area by id' do
          tags TAGS_API_DMO_UNTENSED_AREAS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '200', 'untensed_area found' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:untensed_area) {create(:untensed_area)}
            let(:id) {untensed_area.id}

            include_context 'with integration test'
            include_context 'valid jsonapi ressource' do
              let(:expected_ressource) {untensed_area}
            end
          end
        end
      end

      path '/api/dmo/untensed_areas/{id}' do
        put 'edit untensed_area' do
          tags TAGS_API_DMO_UNTENSED_AREAS
          produces 'application/json'
          consumes 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]
          json_params = {}
          MUTATE_PARAMS.each do |key, val|
            json_params[key] = {type: nice_type(val)}
          end
          parameter name: :params, in: :body, schema: jsonify_params(json_params)
          #jsonapi_params( **json_params )

          response '200', 'untensed_area updated' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:untensed_area) {create(:untensed_area)}
            let(:id) {untensed_area.id}
            let(:params) do
              jsonapi_attributes(
                 { department: 'String'}
              )
            end
            include_context 'with integration test'
            include_context 'valid jsonapi'
            include_context 'not containings errors'

            after do |_example|
            
              json = JSON.parse(response.body)

              expect(json['data']).to be_a Hash
              expect(json['data']['id']).to_not be nil
              expect(json['data']['type']).to eq 'untensed_areas'
            end
          end
        end
      end

      path '/api/dmo/untensed_areas/{id}' do
        delete 'Delete a single untensed_area by id' do
          tags TAGS_API_DMO_UNTENSED_AREAS
          produces 'application/json'
          parameter name: :id, in: :path, type: :string
          security [Bearer: {}]

          response '204', 'untensed_area deleted' do
            let(:Authorization) {"Bearer #{token_for(admin)}"}
            let(:id) {create(:untensed_area, responsible: admin.id).id}
            run_test!
          end
        end
      end
    end
  end
end